/**
 * CacheFactory.java
 * Created on 2005/05/12
 * Copyright (c) by es , All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND 
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF 
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package izayoi.cache;

import izayoi.Configures;
import izayoi.Izayoi;
import izayoi.Util;
import izayoi.node.NodeController;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;

/**
 * alternate of net.sourceforge.shingetsu.cache.CacheManager
 * @author es
 */
public abstract class CacheFactory {

    private static CacheFactory _instance;
    private NodeController _nodeCtrl;

    protected boolean isMakeThumbNail;
    protected String thumbNailFormat;
	private String thumbNailPath;
	protected String basePath;
	private File thumbStoreDir;

    public CacheFactory() {
        Configures c = Configures.getConfigures();
        isMakeThumbNail = Boolean.valueOf(c.getParameter(Izayoi.CONFIG_MAKE_THUMB, "false")).booleanValue();
        if (isMakeThumbNail) {
            thumbNailPath = c.getParameter(Izayoi.CONFIG_MAKE_THUMB_PATH, "./");
            thumbNailFormat = c.getParameter(Izayoi.CONFIG_MAKE_THUMB_FORMAT, "png");
        }
        init(c);
    }

    /**
     * p[^Ŏw肳ꂽLbVfactory̎擾
     * @return
     */
    synchronized public static final CacheFactory createInstance(Class cacheFactory) {
        if (_instance != null) {
            return null;
        }
        try {
            _instance = (CacheFactory) cacheFactory.getDeclaredConstructor(null).newInstance(null);
            return getInstance();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * LbVfactory̎擾
     * @return
     */
    public static final CacheFactory getInstance() {
        return _instance;
    }

    /**
     * ݔFĂSLbVGg擾
     * @return SLbVGg
     * @throws IOException
     */
    public final CacheEntry[] getEntris() throws IOException {
        return getEntris(null, null);
    }

    /**
     * ݔFĂSLbVGg擾
     * @param sorter \[gNX (nullŃ\[gȂ)
     * @return SLbVGg
     * @throws IOException
     */
    public final CacheEntry[] getEntris(CacheEntrySorter sorter) throws IOException {
        return getEntris(null, sorter);
    }

    /**
     * ݔFĂw^CvɑSLbVGg擾
     * @param type ^Cv (nullŎwȂ)
     * @return w^CvɑSLbVGg
     * @throws IOException
     */
    abstract public CacheEntry[] getEntris(String type) throws IOException;

    /**
     * ݔFĂw^CvɑSLbVGg擾
     * @param type ^Cv (nullŎwȂ)
     * @param sorter \[gNX (nullŃ\[gȂ)
     * @return w^CvɑSLbVGg
     * @throws IOException
     */
    public final CacheEntry[] getEntris(String type, CacheEntrySorter sorter) throws IOException {
        CacheEntry[] e = getEntris(type);
        if (sorter != null) {
            Arrays.sort(e, sorter);
        }
        return e;
    }

    /**
     * LbVGg擾AGg݂Ȃꍇnull
     * @param filename t@C
     * @return vꂽLbṼGg
     * @throws IOException
     */
    public final CacheEntry getEntry(String filename) throws IOException {
        return getEntry(filename, false);
    }

    /**
     * LbVGg擾
     * @param filename t@C
     * @param isCreate Gg݂Ȃꍇɋ̃Gg쐬邩
     * @return
     */
    public final CacheEntry getEntry(String filename, boolean isCreate) throws IOException {
        String[] s = filename.split("_");
        try {
            return getEntry(s[0], Util.parseString(s[1]), isCreate);
        } catch (ArrayIndexOutOfBoundsException e) {
            throw new IOException("Bad filename");
        }
    }

    /**
     * LbVGg擾AGg݂Ȃꍇnull
     * @param type ^Cv
     * @param basename x[Xl[
     * @return vꂽLbṼGg
     * @throws IOException
     */
    public final CacheEntry getEntry(String type, String basename) throws IOException {
        return getEntry(type, basename, false);
    }

    /**
     * LbVGg擾
     * @param type ^Cv
     * @param basename x[Xl[
     * @param isCreate Gg݂Ȃꍇɋ̃Gg쐬邩
     * @return vꂽLbṼGg
     * @throws IOException
     */
    public abstract CacheEntry getEntry(String type, String basename, boolean isCreate) throws IOException;

    /**
     * ݒ肳ĂNodeController擾
     * @param ctrl
     */
    protected final NodeController getNodeController() {
        return _nodeCtrl;
    }

    /**
     * 
     * @param config
     */
    abstract public void init(Configures config);

    /**
     * LbVGg݂邩ۂ
     * @param type ^Cv
     * @param basename x[Xl[
     * @return
     */
    abstract public boolean isExist(String type, String basename) throws IOException;

    /**
     * kډ摜t邩?
     * @return
     */
    public final boolean isMakeThumbNail() {
        return isMakeThumbNail;
    }

    /**
     * NodeɍXVʒm
     * @param ent XVꂽGg
     * @param record XVꂽR[h
     * @throws IOException
     */
    public final void noticeUpdate(CacheEntry ent, Record record) throws IOException {
        if (_nodeCtrl != null) {
            String id = record.isDeletedRecord() ? record.getDeleteRequestID() : record.getID();
            long time = record.isDeletedRecord() ? record.getDeletedTime() : record.getTimeStamp();
            _nodeCtrl.sendUpdate(null, ent.getFileName(), time, id);
        }
    }

    /**
     * NodepNodeControllerݒ
     * @param ctrl
     */
    synchronized public final void registNodeController(NodeController ctrl) {
        _nodeCtrl = ctrl;
    }

    /**
     * @param file
     */
    public void setBasePath(String file) {
        basePath = file;
    }

    /**
     * @return
     */
    public File getThumbStorePath() {
    	if(thumbStoreDir == null) {
			thumbStoreDir = new File(basePath, thumbNailPath);
    	}
        return thumbStoreDir;
    }

}
