package izayoi.node.impl.xvi;

import izayoi.Configures;
import izayoi.Izayoi;
import izayoi.cache.CacheFactory;
import izayoi.node.GenericNode;
import izayoi.node.Node;
import izayoi.node.NodeAdapter;
import izayoi.node.NodeController;
import izayoi.node.TalkingEvent;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import edu.emory.mathcs.backport.java.util.concurrent.ConcurrentHashMap;

/**
 * alternate of net.sourceforge.shingetsu.node.NodeList
 * @author es
 */
public class XVInodeController implements NodeController {

    private CacheFactory _cache;
    private String myURL;
    private Map nodeMap = new ConcurrentHashMap(4, 0.75f, 16);

    /**
     * 
     */
    public XVInodeController() {
        Configures conf = Configures.getConfigures();
        myURL = conf.getParameter(Izayoi.CONFIG_SELF_NODE);
    }

    /* ( Javadoc)
     * @see izayoi.node.NodeController#receiveUpdate(java.net.URL, long, java.lang.String, java.lang.String)
     */
    public void sendUpdate(URL sender, String filename, long time, String id) throws IOException {
        Node from = (sender == null) ? null : getNode(sender);
        List nodes = getAllNodes();
        for (Iterator i = nodes.iterator(); i.hasNext();) {
            GenericNode node = (GenericNode) i.next();
            if (node.equals(from)) {
                continue;
            }
            node.asyncUpdate(filename, Long.toString(time), id, getSelfURL());
        }
    }

    /* ( Javadoc)
     * @see izayoi.node.NodeController#sendPing()
     */
    public void sendPing() throws IOException {
        List nodes = getAllNodes();
        for (Iterator i = nodes.iterator(); i.hasNext();) {
            GenericNode node = (GenericNode) i.next();
            node.addNodeListener(new AsyncAgent(null));
            node.asyncPing();
        }
    }

    /* ( Javadoc)
     * @see izayoi.node.NodeController#disconnect()
     */
    synchronized public void disconnect() throws IOException {
        List nodes = getAllNodes();
        for (Iterator i = nodes.iterator(); i.hasNext();) {
            GenericNode node = (GenericNode) i.next();
            node.bye();
        }
    }

    /* ( Javadoc)
     * @see izayoi.node.NodeController#addNode(java.net.URL)
     */
    public void addNode(URL nodeUrl) {
        if (!nodeMap.containsKey(nodeUrl)) {
            nodeMap.put(nodeUrl, new XVInode(nodeUrl, myURL));
        }
    }

    /* ( Javadoc)
     * @see izayoi.node.NodeController#removeNode(java.net.URL)
     */
    public void removeNode(URL nodeUrl) {
        nodeMap.remove(nodeUrl);
    }

    /* ( Javadoc)
     * @see izayoi.node.NodeController#containsNode(java.net.URL)
     */
    public boolean containsNode(URL nodeUrl) {
        return nodeMap.containsKey(nodeUrl);
    }

    /* ( Javadoc)
     * @see izayoi.node.NodeController#checkExist(java.net.URL)
     */
    public boolean checkExist(URL url) {
        try {
            return new XVInode(url, myURL).ping();
        } catch (Exception e) {
            return false;
        }
    }

    /* ( Javadoc)
     * @see izayoi.node.NodeController#getNode(java.net.URL)
     */
    public Node getNode(URL nodeUrl) {
        return (Node) nodeMap.get(nodeUrl);
    }

    /* ( Javadoc)
     * @see izayoi.node.NodeController#getRandomNode()
     */
    public Node getRandomNode() {
        if (nodeMap.size() == 0) {
            return null;
        }
        int num = (int) (Math.random() * nodeMap.size());
        return (Node) nodeMap.values().toArray()[num];
    }

    /* ( Javadoc)
     * @see izayoi.node.NodeController#getAllNodes()
     */
    public List getAllNodes() {
        return new ArrayList(nodeMap.values());
    }

    /* ( Javadoc)
     * @see izayoi.node.NodeController#getSelfURL()
     */
    public String getSelfURL() {
        return myURL;
    }

    /* ( Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return this.getClass().getName();
    }

    /* ( Javadoc)
     * @see izayoi.node.NodeController#registCacheFactory(izayoi.cache.CacheFactory)
     */
    synchronized public void registCacheFactory(CacheFactory cache) {
        _cache = cache;
    }

    /**
     * @return
     */
    protected CacheFactory getCacheFactory() {
        return _cache;
    }

    //---------------------------------------------------------------------

    class AsyncAgent extends NodeAdapter {
        private String filename;

        AsyncAgent(String filename) {
            this.filename = filename;
        }

        /* ( Javadoc)
         * @see izayoi.node.NodeAdapter#ping(izayoi.node.TalkingEvent)
         */
        public void ping(TalkingEvent event) {
            try {
                if (!event.getResult()) {
                    removeNode(event.getSourceNode().getServantURL());
                }
            } finally {
                event.getSourceNode().removeNodeListener(this);
            }
        }
    }

}