package izayoi.servlet;

import izayoi.Izayoi;
import izayoi.cache.CacheEntry;
import izayoi.cache.Record;
import izayoi.exception.InvalidParameter;
import izayoi.exception.NotImplementedException;
import izayoi.node.Node;

import java.io.IOException;
import java.io.PrintWriter;
import java.net.MalformedURLException;
import java.net.URL;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * hserver.cgih
 * @author es
 */
public class ServerServlet extends BasementServlet {

    private static final String schme = "http://";

    protected void execute(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException {

        String[] arg = this.parseURL(req);
        if (arg == null) {
            res.sendError(HttpServletResponse.SC_NOT_FOUND);
        }
        String cmd = arg[0] == null ? "/" : arg[0];
        String line = req.getRequestURI();
        String argument = line.substring(line.indexOf(req.getServletPath()) + req.getServletPath().length());
        argument = argument.substring(argument.indexOf(cmd) + cmd.length());
        if (argument.length() > 1) {
            argument = argument.substring(1);
        }

        res.setContentType("text/plain; charset=UTF-8");
        if (cmd.equals("ping")) {
            ping(req, res);
        } else if (cmd.equals("join")) {
            join(res, argument);
        } else if (cmd.equals("node")) {
            node(res);
        } else if (cmd.equals("bye")) {
            bye(res, argument);
        } else if (cmd.equals("have")) {
            have(res, argument);
        } else if (cmd.equals("get")) {
            get(res, argument);
        } else if (cmd.equals("head")) {
            head(res, argument);
        } else if (cmd.equals("update")) {
            update(res, argument, req.getRemoteAddr());
        } else if (cmd.equals("/")) {
            root(res);
        } else {
            throw new NotImplementedException(cmd);
        }
    }

    /*
     * "ping"̎M
     * @param res
     */
    private void ping(HttpServletRequest req, HttpServletResponse res) throws IOException {
        log("Received ping");
        PrintWriter w = res.getWriter();
        w.println("PONG");
        w.println(req.getRemoteAddr());
    }

    /*
     * "join"̎M
     * @param res
     * @param argument
     */
    private void join(HttpServletResponse res, String argument) throws IOException {
        log("Received join :" + argument);
        argument = argument.replaceAll("\\+", "/");
        URL url = new URL(schme + argument);

        PrintWriter w = res.getWriter();
        if (node.checkExist(url)) {
            w.println("WELCOME");
            node.addNode(url);
        } else {
            w.println("GET OUT");
        }
    }

    /*
     * "node"̎M
     * @param res
     */
    private void node(HttpServletResponse res) throws IOException {
        Node n = node.getRandomNode();
        String s;
        if (n == null) {
            s = conf.getParameter(Izayoi.CONFIG_SELF_NODE, "");
        } else {
            URL url = n.getServantURL();
            s = url.getHost() + ":" + url.getPort() + url.getFile();
        }
        PrintWriter w = res.getWriter();
        w.println(s);
    }

    /*
     * "bye"̎M
     * @param res
     * @param argument
     */
    private void bye(HttpServletResponse res, String argument) throws IOException {
        log("Received bye :" + argument);
        PrintWriter w = res.getWriter();
        argument = argument.replaceAll("\\+", "/");
        URL url = new URL(schme + argument);

        w.println("BYEBYE");
        node.removeNode(url);
    }

    /*
     * "have"̎M
     * @param res
     * @param argument
     */
    private void have(HttpServletResponse res, String argument) throws IOException {
        log("Received have :" + argument);
        PrintWriter w = res.getWriter();
        boolean ret;
        try {
            CacheEntry e = cache.getEntry(argument);
            ret = (e != null);
        } catch (Exception e) {
            ret = false;
        }
        w.println(ret ? "YES" : "NO");
    }

    /*
     * "get"̎M
     * @param res
     * @param argument
     */
    private void get(HttpServletResponse res, String argument) throws IOException, ServletException {
        log("Received get :" + argument);
        String filename = null;
        String time = null;
        try {
            filename = argument.substring(0, argument.indexOf("/"));
            time = argument.substring(argument.indexOf("/") + 1);
        } catch (Exception e1) {
            throw new InvalidParameter("argument", e1.getMessage());
        }

        CacheEntry ent;
        try {
            ent = cache.getEntry(filename);
            if (ent == null) {
                return;
            }
            PrintWriter w = res.getWriter();
            Record[] ret = (time != null && !time.equals("")) ? ent.getRecords(time) : ent.getRecords();
            for (int i = 0; i < ret.length; i++) {
                w.println(ret[i].parseString());
            }
        } catch (Exception e) {
            throw new ServletException(e);
        }
    }

    /*
     * "head"̎M
     * @param res
     * @param argument
     */
    private void head(HttpServletResponse res, String argument) {
        log("Received head :" + argument);
        String filename = argument.substring(0, argument.indexOf("/"));
        String time = argument.substring(argument.indexOf("/") + 1);
        CacheEntry ent;
        try {
            ent = cache.getEntry(filename);
            if (ent == null) {
                return;
            }
            if (time == null || time.equals("")) {
                time = "0-";
            }
            String[] ret = ent.head(time);

            PrintWriter w = res.getWriter();
            for (int i = 0; i < ret.length; i++) {
                w.println(ret[i]);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return;
        }
    }

    /*
     * hupdateh̎M
     * @param res
     * @param argument
     * @param host
     */
    private void update(HttpServletResponse res, String argument, String host) throws IOException {
        log("Received update :" + argument);
        try {
            String[] arg = argument.split("/");
            URL server = parseServerUrl(arg[3], host);
            node.sendUpdate(server, arg[0], Long.parseLong(arg[1]), arg[2]);
            client.syncUpdate(server, arg[0], Long.parseLong(arg[1]), arg[2]);
        } catch (Exception e) {
            if (e instanceof IOException) {
                throw (IOException) e;
            } else if (
                e instanceof ArrayIndexOutOfBoundsException
                    || e instanceof NumberFormatException
                    || e instanceof MalformedURLException) {
                res.sendError(HttpServletResponse.SC_BAD_REQUEST);
                return;
            } else {
                e.printStackTrace();
            }
        }
    }

    /*
     * @param string
     * @param host
     * @return
     */
    private URL parseServerUrl(String string, String host) throws MalformedURLException {
        if (string.startsWith(":") || string.startsWith("+")) {
            string = host + string;
        }
        string = string.replaceAll("\\+", "/");
        return new URL(schme + string);
    }

    /*
     * "/"̎M
     * @param res
     */
    private void root(HttpServletResponse res) throws IOException {
        log("Received root");
        PrintWriter w = res.getWriter();
        w.println(Izayoi.VERSION);
        w.println("NodeController : " + node.toString());
        w.println("CacheController : " + cache.toString());
        w.println("Number of nodes : " + node.getAllNodes().size());
    }

}
