package izayoi.servlet;

import izayoi.Constants;
import izayoi.Util;
import izayoi.cache.CacheEntry;
import izayoi.cache.GenericRecord;
import izayoi.cache.Record;
import izayoi.cache.RecordSorter;
import izayoi.cache.WritableRecord;
import izayoi.exception.InvalidParameter;
import izayoi.view.DataContainer;
import izayoi.view.TemplateContext;
import izayoi.view.ViewThread;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.prixdami.alexis.multipart.MultipartFile;
import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;

/**
 * Replace of ThreadCGI.java
 * @author es
 */
public class ThreadServlet extends BasementServlet {

    /* ( Javadoc)
     * @see izayoi.servlet.BasementServlet#execute(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    protected void execute(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        String[] reqline = parseURL(request);
        if (reqline == null) {
            response.sendError(HttpServletResponse.SC_NOT_FOUND);
            return;
        }

        String command = request.getParameter(PARAM_COMMAND);
        String threadName = reqline[0];
        String attatchID = reqline[1];
        String attatchFilename = reqline[2];

        if (threadName == null || threadName.equals("")) {
            response.sendError(HttpServletResponse.SC_NOT_FOUND);
            return;
        } else {
            threadName = Util.urlDecode(threadName);
        }

        if (request.getMethod().equalsIgnoreCase(COMMAND_POST) && command != null && command.equals(COMMAND_POST)) {
            //e
            postMessage(request, response, threadName);
        } else if (attatchID != null && attatchFilename != null) {
            //Yt擾
            getAttach(request, response, cache.getEntry(Constants.TYPE_THREAD, threadName), attatchID, attatchFilename);
        } else {
            //R[h\
            getRecord(request, response, threadName);
        }

    }

    /**
     * Yt擾
     * @param request
     * @param response
     * @param entry Gg
     * @param attatchID ʎq
     * @param attatchFilename t@C
     */
    private void getAttach(
        HttpServletRequest request,
        HttpServletResponse response,
        CacheEntry entry,
        String attatchID,
        String attatchFilename)
        throws IOException, ServletException {

        int pos = attatchFilename.indexOf(".");
        if (entry == null || pos == -1) {
            response.sendError(HttpServletResponse.SC_NOT_FOUND);
            return;
        }

        String time = attatchFilename.substring(0, pos);
        String ext = attatchFilename.substring(pos + 1);
        Record record = entry.getRecord(Long.parseLong(time), attatchID);
        if (record == null
            || !record.containsField(Constants.FIELD_NAME_ATTACH)
            || !record.containsField(Constants.FIELD_NAME_SUFFIX)) {
            response.sendError(HttpServletResponse.SC_NOT_FOUND);
            return;
        }

        String mime = Util.getMimeType(attatchFilename);
        if (mime == null) {
            mime = "application/octet-stream";
        }
        response.setContentType(mime);
        if (mime.equals("application/octet-stream")) {
            response.setHeader("content-dispotion", "attachment; filename=" + attatchFilename);
        } else {
            response.setHeader("content-dispotion", "inline; filename=" + attatchFilename);
        }

        try {
            InputStream in = new StringInputStream(record.getField(Constants.FIELD_NAME_ATTACH));
            BASE64Decoder dec = new BASE64Decoder();
            dec.decodeBuffer(in, response.getOutputStream());
        } catch (IOException e) {
            throw new ServletException("t@CjĂ܂", e);
        }
        response.flushBuffer();
    }

    /**
     * R[h\
     * @param request
     * @param response
     * @param basename Ώۂ̃x[Xl[
     */
    private void getRecord(HttpServletRequest request, HttpServletResponse response, String basename)
        throws ServletException, IOException {
        CacheEntry entry = cache.getEntry(Constants.TYPE_THREAD, basename);
        String filename = (entry == null) ? basename : entry.getBaseName();

        //\[g擾
        RecordSorter sorter = null;
        String order = (String) request.getAttribute(PARAM_ORDER_THREAD);
        String temp = request.getParameter(PARAM_ORDER);
        if (temp != null) {
            order = temp;
        }
        log(order);
        if (order == null) {
			order = Constants.SORT_ASC;
        }
		sorter = new RecordSorter.ByTimestamp();
		sorter.setOrder(!order.equals(Constants.SORT_DESC));

        //\擾
        int getCount = 0;
        Integer val = (Integer) request.getAttribute(PARAM_LAST);
        if (val != null) {
            getCount = val.intValue();
        }
        temp = request.getParameter(PARAM_LAST);
        if (temp != null) {
            try {
                getCount = Integer.parseInt(temp);
            } catch (NumberFormatException e) {
            }
        }

        //Yt\L擾
        boolean needThumb = true;
        Boolean b = (Boolean) request.getAttribute(PARAM_NEED_THUMB);
        if (b != null) {
            needThumb = b.booleanValue();
        }
        temp = request.getParameter(PARAM_NEED_THUMB);
        if (temp != null) {
            needThumb = Boolean.valueOf(temp).booleanValue();
        }

        //R[h擾
        Record[] record;
        if (entry == null) {
            record = new Record[0];
        } else if (getCount == 0) {
            record = entry.getRecords(sorter);
        } else {
            record = entry.getRecords(getCount, sorter);
        }

        DataContainer[] data = ViewThread.parseEntry(record, request.getContextPath(), Constants.TYPE_THREAD, needThumb);
        TemplateContext ctx = new TemplateContext("thread.txt", request, response);
        ctx.setRecord(data);
        ctx.setName(filename);
        ctx.setSize(entry != null ? entry.getSize() : 0);
        ctx.finish();
    }

    /**
     * e
     * @param request
     * @param response
     * @param target Ώۂ̃x[Xl[
     */
    private void postMessage(HttpServletRequest request, HttpServletResponse response, String target)
        throws IOException, ServletException {
        MultipartFile[] attach = (MultipartFile[]) request.getAttribute(MultipartFile.MULTIPART_ITEMS);
        try {
            String name = request.getParameter(PARAM_NAME);
            String mail = request.getParameter(PARAM_MAIL);
            String signature = request.getParameter(PARAM_SIGN);
            String message = request.getParameter(PARAM_MESSAGE);
            boolean isNotice = (request.getParameter(PARAM_DOPOST) != null);

            message = Util.escape(message);
            mail = Util.escape(mail);
            name = Util.escape(name);

            if ((message == null || message.equals("")) && (attach == null || attach.length == 0)) {
                throw new InvalidParameter("{/Yt", "l܂");
            }

            WritableRecord record = new GenericRecord();
            record.setTimeStamp(System.currentTimeMillis() / 1000);
            record.setField(Constants.FIELD_NAME_BODY, message);
            record.setField(Constants.FIELD_NAME_NAME, name);
            record.setField(Constants.FIELD_NAME_MAIL, mail);

            //@TODO ֘A

            if (attach != null && attach.length > 0) {
                record = setAttach(attach[0], record);
            }

            CacheEntry entry = cache.getEntry(Constants.TYPE_THREAD, target, true);
            entry.append(record);
            if (!entry.isAutoCommit()) {
                entry.commit();
            }
            if (isNotice) {
                try {
                    entry.noticeMe(record);
                } catch (IOException e1) { //Ȃ
                }
            }

            response.sendRedirect(request.getRequestURI() + "#" + record.getID().substring(0, 8));
        } finally {
            if (attach != null) {
                for (int i = 0; i < attach.length; i++) {
                    attach[i].getFile().delete();
                }
            }
        }
    }

    /*
     * YtWJ
     * @param file
     * @param r
     * @return
     */
    private WritableRecord setAttach(MultipartFile file, WritableRecord record) throws IOException {
        BASE64Encoder enc = new BASE64Encoder();
        OutputStream out = new StringOutputStream();

        enc.encodeBuffer(file.getInputStream(), out);
        record.setField(Constants.FIELD_NAME_ATTACH, out.toString());

        String ext;
        if (file.getOriginFileName().lastIndexOf(".") == -1) {
            ext = "bin";
        } else {
            ext = file.getOriginFileName().substring(file.getOriginFileName().lastIndexOf(".") + 1).toLowerCase();
        }
        record.setField(Constants.FIELD_NAME_SUFFIX, ext);
        return record;
    }
}

//---------------------------------------------------------------------------

class StringInputStream extends InputStream {
    int pos = 0;
    String src;
    StringInputStream(String src) {
        this.src = src;
    }
    public int read() throws IOException {
        if (pos >= src.length()) {
            return -1;
        }
        return src.charAt(pos++);
    }
}

class StringOutputStream extends ByteArrayOutputStream {
    public void write(byte[] b, int s, int e) {
        for (int i = s; i < e; i++) {
            write(b[i]);
        }
    }
    public void write(int i) {
        if (i != 13 && i != 10) {
            super.write(i);
        }
    }
}
